<?php
/**
 * PHP Wrapper for simplified use of the php_rrdtool Extension
 *
 * @author Benny Baumann (BenBE) <BenBE@geshi.org>
 * @version $Id$
 * @copyright 2009-2012 by Benny Baumann
 */

//Datasource Types
define('RRD_DS_ABSOLUTE', 'ABSOLUTE');
define('RRD_DS_COMPUTE', 'COMPUTE');
define('RRD_DS_COUNTER', 'COUNTER');
define('RRD_DS_DERIVE', 'DERIVE');
define('RRD_DS_GAUGE', 'GAUGE');

//Round Robin Archive Consolidation Function Types
define('RRD_RRA_CF_AVG', 'AVERAGE');
define('RRD_RRA_CF_LAST', 'LAST');
define('RRD_RRA_CF_MAX', 'MAX');
define('RRD_RRA_CF_MIN', 'MIN');

define('RRD_RRA_ABD_DEVPREDICT', 'DEVPREDICT');
define('RRD_RRA_ABD_DEVSEASONAL', 'DEVSEASONAL');
define('RRD_RRA_ABD_FAILURES', 'FAILURES');
define('RRD_RRA_ABD_HWPREDICT', 'HWPREDICT');
define('RRD_RRA_ABD_MHWPREDICT', 'MHWPREDICT');
define('RRD_RRA_ABD_SEASONAL', 'SEASONAL');

//Parameter names used by this Wrapper
define('RRD_PARAM_CF_MAX', 'max');
define('RRD_PARAM_CF_MIN', 'min');

define('RRD_PARAM_RRA_XFF', 'XFF');
define('RRD_PARAM_RRA_STEPS', 'STEPS');
define('RRD_PARAM_RRA_COUNT', 'COUNT');

//Default Time Intervals given in Seconds
define('RRD_TIME_SECOND', 1);
define('RRD_TIME_MINUTE', 60*RRD_TIME_SECOND);
define('RRD_TIME_HOUR', 60*RRD_TIME_MINUTE);
define('RRD_TIME_DAY', 24*RRD_TIME_HOUR);
define('RRD_TIME_WEEK', 7*RRD_TIME_DAY);
define('RRD_TIME_MONTH', 30*RRD_TIME_DAY);
define('RRD_TIME_YEAR', 365*RRD_TIME_DAY);

//Some default values for certain intervals
define('RRD_DEFAULT_STEP', 5*RRD_TIME_MINUTE);
define('RRD_DEFAULT_HEARTBEAT', 2*RRD_DEFAULT_STEP);

define('RRD_GRAPH_PLOT_LINE', 'LINE');
define('RRD_GRAPH_PLOT_AREA', 'AREA');

/**
 * RRDTool
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool {
    public static function Archive() {
        return new RRDTool_File();
    }
    public static function Graph() {
        return new RRDTool_Graph();
    }
    public static function Fetch(RRDTool_File $file, $options) {
        return $file->Fetch($options);
    }
    public static function Update(RRDTool_File $file, $time, array $data) {
        return $file->Update($time, $data);
    }
    public static function Info(RRDTool_File $file) {
        return $file->Info('');
    }

    public static function isAvailable() {
        return
            extension_loaded('rrdtool') &&
            function_exists('rrd_clear_error') &&
            function_exists('rrd_create') &&
            function_exists('rrd_dump') &&
            function_exists('rrd_error') &&
            function_exists('rrd_fetch') &&
            function_exists('rrd_graph') &&
            function_exists('rrd_info') &&
            function_exists('rrd_last') &&
            function_exists('rrd_update') &&
            function_exists('rrdtool_info') &&
            function_exists('rrdtool_logo_guid');
    }
    public static function RRDToolInfo() {
        return rrdtool_info();
    }
    public static function Logo() {
        return rrdtool_logo_guid();
    }
    public static function getLastError() {
        return rrd_error();
    }
    public static function clearLastError() {
        return rrd_clear_error();
    }
}

/**
 * RRDTool_File
 *
 * Describes an .RRD file
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_File {

    protected $filename = false;

    protected $Step = RRD_DEFAULT_STEP;

    protected $DS = array();

    protected $RRA = array();

    public function Load($filename) {
        if(!is_readable($filename)) {
            return false;
        }

        $info = $this->Info($filename);

        if(!is_array($info)) {
            return false;
        }

        $this->Step = $info['step'];

        $this->DS = array();
        foreach($info['ds'] as $ds_name => $ds_info) {
            $this->addDS(RRDTool_DataSource::createDSfromInfo($ds_name, $ds_info));
        }

        $this->RRA = array();
        foreach($info['rra'] as $rra_idx => $rra_info) {
            $this->addRRA(RRDTool_Archive::createRRAfromInfo($rra_info));
        }

        $this->filename = $filename;

        return true;
    }

    public function Save($filename) {
        $opts = array();

        $opts[] = '--start=' . $this->Start;
        $opts[] = '--step=' . (int)$this->Step;

        foreach($this->DS as $DSinst) {
            $opts[] = (string)$DSinst;
        }

        foreach($this->RRA as $RRAinst) {
            $opts[] = (string)$RRAinst;
        }

        $a = rrd_create($filename, $opts);

        if($a) {
            $a = $this->Load($filename);
        }

        return $a;
    }

    public function Dump($filename = false) {
        if(!is_readable($filename)) {
            $filename = $this->filename;
            if(!is_readable($filename)) {
                return false;
            }
        }

        return rrd_dump($filename);
    }

    public function Info($filename = false) {
        if(!is_readable($filename)) {
            $filename = $this->filename;
            if(!is_readable($filename)) {
                return false;
            }
        }

        $a = rrd_info($filename);
        if(!is_array($a)) {
            return false;
        }

        $b = array();

        foreach($a as $a_key => $a_value) {
            if(preg_match("/^ds\[(.*?)\]\.(.*)$/", $a_key, $m)) {
                $b['ds'][$m[1]][$m[2]] = $a_value;
            } else if(preg_match("/^rra\[(\d+)\]\.(.*?)\[(\d+)\]\.(.*)$/", $a_key, $m)) {
                $b['rra'][$m[1]][$m[2]][$m[3]][$m[4]] = $a_value;
            } else if(preg_match("/^rra\[(\d+)\]\.(.*)$/", $a_key, $m)) {
                $b['rra'][$m[1]][$m[2]] = $a_value;
            } else {
                $b[$a_key] = $a_value;
            }
        }

        return $b;
    }

    public function Update($time, array $data) {
        if(!is_writable($this->filename)) {
echo "#".$this->filename;
            return false;
        }

        $opts = $time . ':';

        $val = array_fill(0, $this->getDSCount(), 'U');
        foreach($data as $data_ds => $data_val) {
            if(!is_numeric($data_val) || is_nan($data_val)) {
                $data_val = 'U';
            }

            if(is_numeric($data_ds)) {
                $data_ds_ref = $data_ds;
            } else {
                $data_ds_ref = $this->getDS($data_ds);
                if(isset($data_ds_ref)) {
                    $data_ds_ref = $data_ds_ref->Index;
                } else {
                    return false;
                }
            }

            if(array_key_exists($data_ds, $val)) {
                return false;
            }

            $val[$data_ds_ref] = $data_val;
        }

        $opts .= implode(':', $val);

        return rrd_update($this->filename, $opts);
    }

    public function Fetch($Options) {
        if(!is_readable($this->filename)) {
            return false;
        }

        return rrd_fetch($this->filename, $Options);
    }

    public function addDS(RRDTool_DataSource $DS) {
        return $this->DS[$DS->Name] = $DS;
    }

    public function addRRA(RRDTool_Archive $RRA) {
        return $this->RRA[] = $RRA;
    }

    public function removeDS(RRDTool_DataSource $DS) {
        return false;
    }

    public function removeRRA(RRDTool_Archive $RRA) {
        return false;
    }

    public function getDS($index) {
        return $this->DS[$index];
    }

    public function getRRA($index) {
        return $this->RRA[$index];
    }

    public function getDSCount() {
        return count($this->DS);
    }

    public function getRRACount() {
        return count($this->RRA);
    }

    public function getPDPIC($time, $interval = 0) {
        if($interval < 1) {
            $interval = $this->Step;
        }

        return ceil($time / $interval);
    }
}

/**
 * RRDTool_Archive
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
abstract class RRDTool_Archive {

    protected static $RegisteredTypes = array();

    public static function registerType($Type, $Class) {
        self::$RegisteredTypes[$Type] = $Class;
    }

    public static function unregisterType($Type) {
        unset(self::$RegisteredTypes[$Type]);
    }

    public static function createRRA($Type, array $Options) {
        $TypeReg = self::$RegisteredTypes;
        if (!isset($TypeReg[$Type])) {
            return false;
        }

        $Class = $TypeReg[$Type];

        $RRA = new $Class($Options);

        return $RRA;
    }

    public static function createRRACF($Type, $XFF, $Steps, $Samples) {
        return RRDTool_Archive::createRRA(
            $Type,
            array(
                RRD_PARAM_RRA_XFF => $XFF,
                RRD_PARAM_RRA_STEPS => $Steps,
                RRD_PARAM_RRA_COUNT => $Samples
                )
            );
    }

    public static function createRRAfromInfo(array $Info) {
        return RRDTool_Archive::createRRA(
            $Info['cf'],
            array(
                RRD_PARAM_RRA_XFF => $Info['xff'],
                RRD_PARAM_RRA_STEPS => $Info['pdp_per_row'],
                RRD_PARAM_RRA_COUNT => $Info['rows']
                )
            );
    }

}

class RRDTool_Archive_CF_Avg extends RRDTool_Archive {

    protected $XFF = 0.5;
    protected $Steps = 1;
    protected $PDP = 1;

    public function __construct(array $Options) {
        $this->XFF = $Options[RRD_PARAM_RRA_XFF];
        $this->Steps = $Options[RRD_PARAM_RRA_STEPS];
        $this->PDP = $Options[RRD_PARAM_RRA_COUNT];
    }

    public function __toString() {
        return sprintf("RRA:AVERAGE:%f:%d:%d", $this->XFF, $this->Steps, $this->PDP);
    }
}

class RRDTool_Archive_CF_Last extends RRDTool_Archive {

    protected $XFF = 0.5;
    protected $Steps = 1;
    protected $PDP = 1;

    public function __construct(array $Options) {
        $this->XFF = $Options[RRD_PARAM_RRA_XFF];
        $this->Steps = $Options[RRD_PARAM_RRA_STEPS];
        $this->PDP = $Options[RRD_PARAM_RRA_COUNT];
    }

    public function __toString() {
        return sprintf("RRA:LAST:%f:%d:%d", $this->XFF, $this->Steps, $this->PDP);
    }
}

class RRDTool_Archive_CF_Max extends RRDTool_Archive {

    protected $XFF = 0.5;
    protected $Steps = 1;
    protected $PDP = 1;

    public function __construct(array $Options) {
        $this->XFF = $Options[RRD_PARAM_RRA_XFF];
        $this->Steps = $Options[RRD_PARAM_RRA_STEPS];
        $this->PDP = $Options[RRD_PARAM_RRA_COUNT];
    }

    public function __toString() {
        return sprintf("RRA:MAX:%f:%d:%d", $this->XFF, $this->Steps, $this->PDP);
    }
}

class RRDTool_Archive_CF_Min extends RRDTool_Archive {

    protected $XFF = 0.5;
    protected $Steps = 1;
    protected $PDP = 1;

    public function __construct(array $Options) {
        $this->XFF = $Options[RRD_PARAM_RRA_XFF];
        $this->Steps = $Options[RRD_PARAM_RRA_STEPS];
        $this->PDP = $Options[RRD_PARAM_RRA_COUNT];
    }

    public function __toString() {
        return sprintf("RRA:MIN:%f:%d:%d", $this->XFF, $this->Steps, $this->PDP);
    }
}

/**
 * RRDTool_DataSource
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
abstract class RRDTool_DataSource {
    public $Index = -1;

    public $Name;

    protected $Heartbeat = RRD_DEFAULT_HEARTBEAT;

    protected static $RegisteredTypes = array();

    public static function registerType($Type, $Class) {
        self::$RegisteredTypes[$Type] = $Class;
    }

    public static function unregisterType($Type) {
        unset(self::$RegisteredTypes[$Type]);
    }

    public function __construct($Name, $Heartbeat = RRD_DEFAULT_HEARTBEAT) {
        $this->Name = $Name;
        $this->Heartbeat = $Heartbeat;
    }

    public static function createDS($Name, $Type, $Heartbeat, array $Options) {
        $TypeReg = self::$RegisteredTypes;
        if (!isset($TypeReg[$Type])) {
            echo $Type . "\n";
            var_dump($TypeReg);
            return false;
        }

        $Class = $TypeReg[$Type];

        $DS = new $Class($Name, $Heartbeat, $Options);

        return $DS;
    }

    public static function createDSfromInfo($Name, array $Info) {
        //Clean the Options array - most stuff we don't need
        $Options = $Info;
        unset($Options['index']);
        unset($Options['type']);
        unset($Options['minimal_heartbeat']);
        unset($Options['last_ds']);
        unset($Options['value']);
        unset($Options['unknown_sec']);

        //Ask for this Data Source to be created
        $DS = RRDTool_DataSource::createDS(
            $Name,
            $Info['type'],
            $Info['minimal_heartbeat'],
            $Options);

        //Set the internal index according to the info given
        if($DS && isset($Info['index'])) {
            $DS->Index = $Info['index'];
        }

        return $DS;
    }

    public function __toString() {
        return "DS:{$this->Name}:UNKNOWN:{$this->Heartbeat}";
    }
}

/**
 * RRDTool_DataSource_Gauge
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_DataSource_Gauge extends RRDTool_DataSource {

    public function __construct($Name, $Heartbeat, array $Option) {
        parent::__construct($Name, $Heartbeat);
    }

    public function __toString() {
        return "DS:{$this->Name}:".RRD_DS_GAUGE.":{$this->Heartbeat}:U:U";
    }

}

/**
 * RRDTool_DataSource_Counter
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_DataSource_Counter extends RRDTool_DataSource {

    protected $Min = null;
    protected $Max = null;

    public function __construct($Name, $Heartbeat, array $Option) {
        parent::__construct($Name, $Heartbeat);
        $this->Max = $Option['max'];
        $this->Min = $Option['min'];
    }

    public function __toString() {
        return "DS:{$this->Name}:".RRD_DS_COUNTER.":{$this->Heartbeat}:".(null==$this->Min?'U':(int)$this->Min).":".(null==$this->Max?'U':(int)$this->Max);
    }

}

/**
 * RRDTool_DataSource_Absolute
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_DataSource_Absolute extends RRDTool_DataSource {

    protected $Min = null;
    protected $Max = null;

    public function __construct($Name, $Heartbeat, array $Option) {
        parent::__construct($Name, $Heartbeat);
        $this->Max = $Option['max'];
        $this->Min = $Option['min'];
    }

    public function __toString() {
        return "DS:{$this->Name}:".RRD_DS_ABSOLUTE.":{$this->Heartbeat}:".(null==$this->Min?'U':(int)$this->Min).":".(null==$this->Max?'U':(int)$this->Max);
    }
}

/**
 * RRDTool_DataSource_Calculation
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_DataSource_Compute extends RRDTool_DataSource {

    protected $Expression = null;

    public function __construct($Name, $Heartbeat, array $Option) {
        parent::__construct($Name, $Heartbeat);
        $this->Expression = $Option['expression'];
    }

    public function __toString() {
        return "DS:{$this->Name}:".RRD_DS_COMPUTE.":{$this->Heartbeat}:".$this->Expression;
    }

}

/**
 * RRDTool_DataSource_Derive
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_DataSource_Derive extends RRDTool_DataSource {

    protected $Min = null;
    protected $Max = null;

    public function __construct($Name, $Heartbeat, array $Option) {
        parent::__construct($Name, $Heartbeat);
        $this->Max = $Option['max'];
        $this->Min = $Option['min'];
    }

    public function __toString() {
        return "DS:{$this->Name}:".RRD_DS_DERIVE.":{$this->Heartbeat}:".(null==$this->Min?'U':(int)$this->Min).":".(null==$this->Max?'U':(int)$this->Max);
    }

}

/**
 * RRDTool_Graph
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_Graph {

    public $Title;

    public $Width = 640;
    public $Height = 480;

    public $DSData;
    public $DSValue;

    public $Plot;
    public $Legend;
}

class RRDTool_Graph_DSD {
    public $Name;
}

class RRDTool_Graph_DSD_RRA extends RRDTool_Graph_DSD {
    public $File;
    public $DS;
    public $RRDCF;
    public $Step = 0;

    public function __toString() {
        $_Step = $this->Step ? sprintf(":%d", $this->Step) : '';
        return sprintf("DEV:%s=%s:%s:%s", $this->Name, $this->File, $this->DS, $this->RRDCF, $_Step);
    }
}

class RRDTool_Graph_DSD_Expression extends RRDTool_Graph_DSD {

    public function __toString() {
        return sprintf("CDEV:%s=%s:%s:%s", $this->Name, $this->File, $this->DS, $this->RRDCF);
    }
}

class RRDTool_Graph_DSV {
    public $Name;
}

class RRDTool_Graph_DSV_CF extends RRDTool_Graph_DSV {
    public $DSD;
    public $CF;

    public function __toString() {
        return sprintf("VDEV:%s=%s,%s", $this->Name, $this->DSD->Name, $this->CF);
    }
}

class RRDTool_Graph_Plot {
    public $DSD;
    public $Color;
    public $Legend; //Uses a legend class to allow for more fancy texts
    public $Stacked = false;

    public function getGenerateData() {
        $Result = array($this->getCommandData());
        $Legend = $this->getLegendData();
        foreach($Legend as $L) {
            array_push($Result, $Legend);
        }
        return $Result;
    }

    public function getCommandData() {
        $Result = 'UNKNOWN:';
        $Result .= $this->DSD->Name;
        if($this->Color) {
            $Result .= '#'.$this->Color;
        }
        if($this->Legend) {
            $Result .= ':\\g';
        }
        if($this->Stacked) {
            $Result .= ':STACK';
        }
        return $Result;
    }

    public function getLegendData() {
        if(!$this->Legend) {
            return array();
        }

        return $this->Legend->getCommandData();
    }

    public function __toString() {
        return $this->getCommandData();
    }
}

class RRDTool_Graph_Plot_Line extends RRDTool_Graph_Plot {
    public $Width = 0;
    public $Dashes = false; //or true or array with def)
    public $DashOffset = 0;

    public function getCommandData() {
        $Result = 'LINE';
        if($this->Color) {
            $Result .= (int)$this->Width;
        }
        $Result .= ':';
        $Result .= $this->DSD->Name;
        if($this->Color) {
            $Result .= '#'.$this->Color;
        }
        if($this->Legend) {
            $Result .= ':\\g';
        }
        if($this->Stacked) {
            $Result .= ':STACK';
        }
        if($this->Dashes) {
            $Result .= ':dashes';
            if(is_array($this->Dashes)) {
                $Result .= '=' . implode(',', $this->Dashes);
                if($this->DashOffset) {
                    $Result .= sprintf(':dash-offset=%d', $this->DashOffset);
                }
            }
        }
        return $Result;
    }
}

class RRDTool_Graph_Plot_Area extends RRDTool_Graph_Plot {

    public function getCommandData() {
        $Result = 'AREA:';
        $Result .= $this->DSD->Name;
        if($this->Color) {
            $Result .= '#'.$this->Color;
        }
        if($this->Legend) {
            $Result .= ':\\g';
        }
        if($this->Stacked) {
            $Result .= ':STACK';
        }
        return $Result;
    }
}

/**
 * RRDTool_Expression
 *
 * @package RRDTool
 * @author Administrator
 * @copyright Copyright (c) 2009
 * @version $Id$
 * @access public
 */
class RRDTool_Expression {

}

//Register known Data Source types
RRDTool_DataSource::registerType(RRD_DS_GAUGE, "RRDTool_DataSource_Gauge");
RRDTool_DataSource::registerType(RRD_DS_COUNTER, "RRDTool_DataSource_Counter");
RRDTool_DataSource::registerType(RRD_DS_ABSOLUTE, "RRDTool_DataSource_Absolute");
RRDTool_DataSource::registerType(RRD_DS_COMPUTE, "RRDTool_DataSource_Compute");
RRDTool_DataSource::registerType(RRD_DS_DERIVE, "RRDTool_DataSource_Derive");

//Register known Round-Robin Archive types
RRDTool_Archive::registerType(RRD_RRA_CF_AVG, "RRDTool_Archive_CF_Avg");
RRDTool_Archive::registerType(RRD_RRA_CF_LAST, "RRDTool_Archive_CF_Last");
RRDTool_Archive::registerType(RRD_RRA_CF_MAX, "RRDTool_Archive_CF_Max");
RRDTool_Archive::registerType(RRD_RRA_CF_MIN, "RRDTool_Archive_CF_Min");

?>